@@ -1,4 +1,4 @@
-#-*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 
 # Copyright 2012-2016 Mir Calculate. http://www.calculate-linux.org
 #
@@ -35,7 +35,16 @@ from CertificateClass import CertClass
 from pid_information import client_list_pid
 from more import icon_visible, client_del_sid, show_msg
 
-class MainWgt(QtGui.QMainWindow):            
+DEBUG_LEVEL = 0
+
+
+def debug(level, *args):
+    if level <= DEBUG_LEVEL:
+        for s in args:
+            print s,
+        print
+
+class MainWgt(QtGui.QMainWindow):
     def __init__(self, ClientObj):
         QtGui.QMainWindow.__init__(self)
         self.ClientObj = ClientObj
@@ -43,7 +52,7 @@ class MainWgt(QtGui.QMainWindow):
 
     def initUI(self):
         # Create bottom status bar
-#        self.statusbar = self.statusBar()
+        #        self.statusbar = self.statusBar()
 
         self.Main = QtGui.QWidget(self)
 
@@ -70,21 +79,21 @@ class MainWgt(QtGui.QMainWindow):
         self.main_frame = QtGui.QScrollArea(self)
         self.main_frame.setFrameShape(QtGui.QFrame.Shape.NoFrame)
         self.main_frame.setObjectName('MF')
-        self.main_frame.setStyleSheet("#MF " "{background-color: #FFFFFF;}")
+        #self.main_frame.setStyleSheet("#MF " "{background-color: #FFFFFF;}")
 
         self.MainFrameWgt = StartWidget(self)
-        self.main_frame.setWidget (self.MainFrameWgt)
+        self.main_frame.setWidget(self.MainFrameWgt)
         self.main_frame.setWidgetResizable(True)
 
         self.layout.addWidget(self.left_frame, 1, 0, 2, 1)
         self.layout.addWidget(self.main_frame, 1, 1)
         self.layout.addWidget(self.control_button, 2, 1)
-        
-        self.layout.setContentsMargins(0,0,0,0)
+
+        self.layout.setContentsMargins(0, 0, 0, 0)
         self.layout.setSpacing(0)
 
         self.vlayout.addWidget(self.layout_wgt)
-        self.vlayout.setContentsMargins(0,0,0,0)
+        self.vlayout.setContentsMargins(0, 0, 0, 0)
         self.vlayout.setSpacing(0)
 
         self.setCentralWidget(self.Main)
@@ -100,12 +109,12 @@ class MainWgt(QtGui.QMainWindow):
         self.ConnectWidget.initUI(self, self.ClientObj._parent)
         self.ConnectWidget.show()
 
-    def connect_to_localhost(self, host = None, port = None):
+    def connect_to_localhost(self, host=None, port=None):
         self.cur_size = self.size()
         self.ConnectWidget = FrameConnection(self, self.ClientObj)
         if not host:
             host = 'localhost'
-        self.ConnectWidget.connect_to_host(host, port, auto = True)
+        self.ConnectWidget.connect_to_host(host, port, auto=True)
 
     # 3 methods for help menu
     def help(self):
@@ -115,9 +124,9 @@ class MainWgt(QtGui.QMainWindow):
 
     def hand_book(self):
         if self.ClientObj.lang == 'ru':
-            site="http://www.calculate-linux.ru/main/ru/calculate-console-gui"
+            site = "http://www.calculate-linux.ru/main/ru/calculate-console-gui"
         else:
-            site="http://www.calculate-linux.org/main/en/calculate-console-gui"
+            site = "http://www.calculate-linux.org/main/en/calculate-console-gui"
         QtGui.QDesktopServices.openUrl(QtCore.QUrl(site))
 
     def bug_report(self):
@@ -126,12 +135,13 @@ class MainWgt(QtGui.QMainWindow):
         self.bugWidget.show()
 
     def tools(self):
-        self.ToolsWgt = ToolsWidget(self,self.ClientObj,self.ClientObj._parent)
+        self.ToolsWgt = ToolsWidget(self, self.ClientObj,
+                                    self.ClientObj._parent)
         self.ToolsWgt.show()
 
     def work_with_certificates(self):
         self.certWidget = CertClass(self, self.ClientObj, \
-                                            self.ClientObj._parent)
+                                    self.ClientObj._parent)
         self.certWidget.show()
 
     def display_methods(self):
@@ -139,29 +149,30 @@ class MainWgt(QtGui.QMainWindow):
 
         self.MainFrameWgt = DisplayMethod(self, self.ClientObj)
         self.MainFrameWgt.refresh()
-        self.main_frame.setWidget (self.MainFrameWgt)
-        self.main_frame.setWidgetResizable(True)        
+        self.main_frame.setWidget(self.MainFrameWgt)
+        self.main_frame.setWidgetResizable(True)
 
         self.left_frame.hide()
         self.left_frame.old_title = None
 
-#        icon_visible(self, 'Methods', False)
+        #        icon_visible(self, 'Methods', False)
         icon_visible(self, 'Back', False)
         self.resize(self.cur_size)
 
     def back(self):
-        if len (self.ClientObj.back_step_list) < 2:
+        self.control_button.del_button()
+        if len(self.ClientObj.back_step_list) < 2:
             self.delete_MainFrameWgt()
 
             self.MainFrameWgt = DisplayMethod(self, self.ClientObj)
             self.MainFrameWgt.refresh()
-            self.main_frame.setWidget (self.MainFrameWgt)
-            self.main_frame.setWidgetResizable(True)        
+            self.main_frame.setWidget(self.MainFrameWgt)
+            self.main_frame.setWidgetResizable(True)
 
             self.left_frame.hide()
             self.left_frame.old_title = None
 
-#            icon_visible(self, 'Methods', False)
+            #            icon_visible(self, 'Methods', False)
             icon_visible(self, 'Back', False)
 
         else:
@@ -169,30 +180,30 @@ class MainWgt(QtGui.QMainWindow):
             title = self.ClientObj.back_step_list.pop()[2]
             # get group, subgroup
             group, subgroup, temp = self.ClientObj.back_step_list \
-                                    [len(self.ClientObj.back_step_list)-1]
+                [len(self.ClientObj.back_step_list) - 1]
 
             # create new group in left menu
             self.left_frame.groupActivated(group, subgroup, title, \
-                                                        self.ClientObj, True)
+                                           self.ClientObj, True)
 
     def view_processes(self):
         self.delete_MainFrameWgt()
         self.MainFrameWgt = ViewProc(self, self.ClientObj)
-        self.main_frame.setWidget (self.MainFrameWgt)
+        self.main_frame.setWidget(self.MainFrameWgt)
         self.main_frame.setWidgetResizable(True)
 
         self.left_frame.hide()
 
-#        icon_visible(self, 'Methods', True)
+        #        icon_visible(self, 'Methods', True)
         icon_visible(self, 'Back', True)
 
     def view_session_info(self):
         self.SessionWgt = ViewSessionInfo(self, self.ClientObj.client, \
-                                                self.ClientObj._parent)
+                                          self.ClientObj._parent)
         self.SessionWgt.setAttribute(QtCore.Qt.WA_ShowModal)
         self.SessionWgt.show()
 
-    def disconnect(self, new_session = False):
+    def disconnect(self, new_session=False):
         if not self.ClientObj.client:
             return 1
 
@@ -202,7 +213,7 @@ class MainWgt(QtGui.QMainWindow):
             reply = self.show_close_messagebox()
 
         if reply in [QtGui.QMessageBox.No, QtGui.QMessageBox.Yes]:
-            if hasattr (self.ClientObj, 'signaling'):
+            if hasattr(self.ClientObj, 'signaling'):
                 if self.ClientObj.signaling.isRunning():
                     self.ClientObj.signaling.close()
                     self.ClientObj.signaling.wait()
@@ -217,21 +228,21 @@ class MainWgt(QtGui.QMainWindow):
                 self.topmenu.Processes.setEnabled(True)
                 self.topmenu.Session.setEnabled(True)
                 self.topmenu.Back.setEnabled(True)
-            if hasattr (self, 'con_lost_lbl'):
+            if hasattr(self, 'con_lost_lbl'):
                 self.con_lost_lbl.hide()
                 self.con_lost_lbl.close()
 
-            if hasattr (self.ClientObj._parent, 'set_localhost'):
+            if hasattr(self.ClientObj._parent, 'set_localhost'):
                 if self.ClientObj.client.server_host_name in \
-                                        ['127.0.0.1', 'localhost']:
+                        ['127.0.0.1', 'localhost']:
                     self.ClientObj._parent.set_localhost(None)
             else:
                 self.ClientObj._parent.connect()
                 return 0
             self.ClientObj.client = None
-            self.ClientObj._parent.connect_count_changed( \
-                        self.ClientObj.host_name, self.ClientObj.port,0)
-#            icon_visible(self, 'Methods', False)
+            self.ClientObj._parent.connect_count_changed(
+                self.ClientObj.host_name, self.ClientObj.port, 0)
+            #            icon_visible(self, 'Methods', False)
             icon_visible(self, 'Back', False)
             icon_visible(self, 'Disconnect', False)
             icon_visible(self, 'Processes', False)
@@ -239,8 +250,8 @@ class MainWgt(QtGui.QMainWindow):
             icon_visible(self, 'Connect', True)
             icon_visible(self, 'Certificates', True)
             self.process_dict = {}
-            
-            if hasattr (self.ClientObj._parent, 'rename_tab'):
+
+            if hasattr(self.ClientObj._parent, 'rename_tab'):
                 self.ClientObj._parent.rename_tab()
         else:
             return 0
@@ -253,7 +264,7 @@ class MainWgt(QtGui.QMainWindow):
         self.left_frame.hide()
 
         # set new Title
-        self.ClientObj._parent.setWindowTitle (self.ClientObj.Name)
+        self.ClientObj._parent.setWindowTitle(self.ClientObj.Name)
         return 0
 
     def reconnect_to_host(self):
@@ -262,36 +273,39 @@ class MainWgt(QtGui.QMainWindow):
         password = self.ClientObj.client.password
         self.disconnect(True)
 
-        if hasattr (self.ClientObj._parent, 'set_localhost'):
+        if hasattr(self.ClientObj._parent, 'set_localhost'):
             # Reconnet to current host
             self.cur_size = self.size()
             self.ConnectWidget = FrameConnection(self, self.ClientObj)
             self.ConnectWidget.connect_to_host(server_host_name,
-                                      self.ClientObj.port, re_passwd=password)
+                                               self.ClientObj.port,
+                                               re_passwd=password)
 
             if not self.main_frame.isEnabled():
                 self.left_frame.setEnabled(True)
                 self.main_frame.setEnabled(True)
                 self.control_button.setEnabled(True)
 
-    def main_frame_view(self, view, method_name, step_change = False, \
-                        change_offset = False, error_output = True,
-                        skip_options=False):
+    def main_frame_view(self, view, method_name, step_change=False, \
+                        change_offset=False, error_output=True,
+                        skip_options=False, new_step=None):
         if change_offset:
             self.MainFrameWgt = MainFrame(self, self.ClientObj, view, \
                                           method_name, error_output)
-            self.MainFrameWgt.initUI()
+            self.MainFrameWgt.initUI(True)
             return
         self.delete_MainFrameWgt()
-        self.MainFrameWgt = MainFrame(self, self.ClientObj, view, method_name,\
+        self.MainFrameWgt = MainFrame(self, self.ClientObj, view, method_name, \
                                       error_output, skip_options=skip_options)
         self.MainFrameWgt.initUI()
-        self.main_frame.setWidget (self.MainFrameWgt)
+        if new_step:
+            self.MainFrameWgt.refresh_view(int(new_step))
+        self.main_frame.setWidget(self.MainFrameWgt)
         self.main_frame.setWidgetResizable(True)
 
-    def main_frame_res(self, method_name, meth_result, change_offset = False):
+    def main_frame_res(self, method_name, meth_result, change_offset=False):
         if change_offset:
-            TempMainFrameWgt = MainFrameRes(self, self.ClientObj, method_name,\
+            TempMainFrameWgt = MainFrameRes(self, self.ClientObj, method_name,
                                             meth_result, change_offset)
             self.left_frame.hide()
 
@@ -322,7 +336,7 @@ class MainWgt(QtGui.QMainWindow):
         self.delete_MainFrameWgt()
         self.MainFrameWgt = MainFrameRes(self, self.ClientObj,
                                          method_name, meth_result)
-        self.main_frame.setWidget (self.MainFrameWgt)
+        self.main_frame.setWidget(self.MainFrameWgt)
         self.main_frame.setWidgetResizable(True)
 
         self.left_frame.hide()
@@ -341,21 +355,30 @@ class MainWgt(QtGui.QMainWindow):
 
     def setScrollBarVal(self):
         self.main_frame.verticalScrollBar().setSliderPosition \
-                                (self.main_frame.verticalScrollBar().maximum())
+            (self.main_frame.verticalScrollBar().maximum())
 
     def signaling_slot(self, sid, reply):
         if reply == 2:
             # Create ConnectLostLabel
-            if hasattr (self, 'con_lost_lbl'):
+            if hasattr(self, 'con_lost_lbl'):
                 self.con_lost_lbl.hide()
                 self.con_lost_lbl.close()
             self.con_lost_lbl = ConnectLostLabel(_('Connection lost!'), self)
+            host_name = self.ClientObj.client.server_host_name
+            port = self.ClientObj.port
+            if host_name in ('127.0.0.1', 'localhost'):
+                try:
+                    from calculate.lib.utils.dbus_tools import run_dbus_core
+                    run_dbus_core(host_name, port)
+                except ImportError:
+                    pass
+                
             self.topmenu.Processes.setDisabled(True)
             self.topmenu.Session.setDisabled(True)
             self.topmenu.Back.setDisabled(True)
             self.con_lost_lbl.show()
         elif reply == 0:
-            if hasattr (self, 'con_lost_lbl'):
+            if hasattr(self, 'con_lost_lbl'):
                 self.con_lost_lbl.hide()
                 self.con_lost_lbl.close()
             if not self.main_frame.isEnabled():
@@ -365,10 +388,10 @@ class MainWgt(QtGui.QMainWindow):
                 self.topmenu.Processes.setEnabled(True)
                 self.topmenu.Session.setEnabled(True)
                 self.topmenu.Back.setEnabled(True)
-                if hasattr (self.MainFrameWgt, 'button_enabled'):
+                if hasattr(self.MainFrameWgt, 'button_enabled'):
                     self.MainFrameWgt.button_enabled()
         else:
-            if hasattr (self, 'con_lost_lbl'):
+            if hasattr(self, 'con_lost_lbl'):
                 self.con_lost_lbl.hide()
                 self.con_lost_lbl.close()
             self.con_lost_lbl = ConnectLostLabel(
@@ -382,7 +405,7 @@ class MainWgt(QtGui.QMainWindow):
             self.reconnect_to_host()
 
     def resizeEvent(self, event):
-        if hasattr (self, 'con_lost_lbl'):
+        if hasattr(self, 'con_lost_lbl'):
             if self.con_lost_lbl.isVisible():
                 self.con_lost_lbl.resize_widget()
         event.accept()
@@ -392,15 +415,17 @@ class MainWgt(QtGui.QMainWindow):
 
         msgBox = QMessageBox(self)
         msgBox.setWindowTitle(_('Closing session'))
-        msgBox.setText(_("Close this session") +' ' +_('with %s?') \
-                                        %self.ClientObj.host_name + '\t')
+        msgBox.setText(_("Close this session") + ' ' + _('with %s?') \
+                       % self.ClientObj.host_name + '\t')
         if len(list_pid):
             if str(len(list_pid)).endswith('1'):
-                msgBox.setInformativeText(_('Once the session is closed, '\
-                    'data on process %d will be lost!') %len(list_pid))
+                msgBox.setInformativeText(_('Once the session is closed, ' \
+                                            'data on process %d will be lost!') % len(
+                    list_pid))
             if len(list_pid) > 1:
-                msgBox.setInformativeText(_('Once the session is closed, '\
-                    'data on process %d will be lost!') %len(list_pid))
+                msgBox.setInformativeText(_('Once the session is closed, ' \
+                                            'data on process %d will be lost!') % len(
+                    list_pid))
 
         msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No | \
                                   QMessageBox.Cancel)
@@ -423,18 +448,18 @@ class MainWgt(QtGui.QMainWindow):
                 self.topmenu.Processes.setEnabled(True)
                 self.topmenu.Session.setEnabled(True)
                 self.topmenu.Back.setEnabled(True)
-                if hasattr (self, 'con_lost_lbl'):
+                if hasattr(self, 'con_lost_lbl'):
                     self.con_lost_lbl.hide()
                     self.con_lost_lbl.close()
                 # Exit with closing session
-                if hasattr (self.ClientObj, 'signaling'):
+                if hasattr(self.ClientObj, 'signaling'):
                     if self.ClientObj.signaling.isRunning():
                         self.ClientObj.signaling.close()
                         self.ClientObj.signaling.wait()
 
-                if hasattr (self.ClientObj._parent, 'set_localhost'):
+                if hasattr(self.ClientObj._parent, 'set_localhost'):
                     if self.ClientObj.client.server_host_name in \
-                                            ['127.0.0.1', 'localhost']:
+                            ['127.0.0.1', 'localhost']:
                         self.ClientObj._parent.set_localhost(None)
 
                 if reply == QtGui.QMessageBox.Yes:
