@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2012-2013 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2012-2016 Mir Calculate. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ from PySide import QtGui, QtCore
 from sudsds import WebFault
 import logging, OpenSSL
 import ConfigParser, os, urllib2
+import getpass
 
 from calculate.core.client.function import clear as clear_suds_cache
 from session_function import client_post_cert
@@ -265,6 +266,12 @@ class FrameConnection(QtGui.QWidget):
         url = "https://%s:%d/?wsdl" %(host_name, port)
         self.ClientObj.port = port
         _print (url)
+
+        try:
+            from calculate.lib.utils.dbus_tools import run_dbus_core
+            run_dbus_core(host_name, port)
+        except ImportError:
+            pass
         path_to_cert = self.ClientObj.path_to_cert
         try:
             client = Client_suds(url, transport = \
@@ -359,6 +366,10 @@ class FrameConnection(QtGui.QWidget):
                     mess = _('You do not have a certificate. Please generate '
                              'a new request and get the new certificate from '
                              'the server.')
+                    if host_name in ("127.0.0.1", "localhost"):
+                        mess = ("%s %s:\ncl-core -u %s")%(mess,
+                            _("Also you can generate administrator certificate by root command"),
+                            getpass.getuser())
                     show_msg (mess, parent = self)
                 elif e.message == 'VerifyError':
                     pass
@@ -367,6 +378,10 @@ class FrameConnection(QtGui.QWidget):
                           'certificate does not match the server certificate.'
                           ' Please generate a new request and get the new '
                           'certificate from the server.')
+                    if host_name in ("127.0.0.1", "localhost"):
+                        mess = ("%s %s:\ncl-core -u %s")%(mess,
+                            _("Also you can generate administrator certificate by root command"),
+                            getpass.getuser())
                     show_msg (mess, parent = self)
                 else:
                     if e.message or e.args:
